/*
 * tmos - pommax2.c
 * 	version 0.0.0
 * 
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 * Copyright (C) 2009  Marek Piwonski <marek@microster.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/console.h>
#include <tmos/crc16.h>
#include <tmos/jiffies.h>
#include <tmos/port.h>
#include <stdlib.h>
#include <limits.h>
#include <tmos/adc.h>
#include <tmos/watchdog.h>
#include <tmos/serial.h>
#include <tmos/spi.h>
#include <asm/aduc841.h>

/*
 * POMMAX2 use synchronous serial link.
 *
 * Frame:
 * POMMAX2_SYNC <CH_0> <CH_1> ... <CH_MAX> <CHECKSUM>
 *
 * POMMAX2_SYNC		1 1 1 1 1 1 1 1
 * <CH_i>		0 0 x x x x x x  0 0 x x x x x x
 * CHECKSUM		0 0 x x x x x x
 *
 * CHECKSUM is bitwise negation of two's complement sum of CH_i bytes.
 */
#define POMMAX2_SYNC	0xff
#define POMMAX2_STOP	0x00

#define CHANNELS CONFIG_POMMAX2_CHANNELS

#if CONFIG_POMMAX2_SPI
#define pommax2_send(c) spi_xmit(c)
#elif CONFIG_POMMAX2_SERIAL
#define pommax2_send(c) ser_txc(c)
#else
#define pommax2_send(c) do { (void) (c); } while (0)
#endif

void app_main(void) REENTRANT
{
	jiffies_t t, ot;
	uint8_t i, x;
	uint8_t csum;
#if DEBUG
	uint16_t samples = 0;
#endif
	int16_t v;
	uint8_t seq = 0;

	ot = get_jiffies();
	i = 0;
	for (;;) {
		watchdog_refresh();
		t = get_jiffies();

#if DEBUG
		samples++;
#endif
		seq = adc_sync(seq);
		pommax2_send(POMMAX2_SYNC);
		csum = 0;
		for (i = 0; i < CHANNELS; i++) {
			adc_wait_ch(i);
			v = adc_get(i);
#if DEBUG
			if (i == 0)
				v = samples;
#endif
			
			/*
			 * send 6 bits per character,
			 * max 12 bit resolution
			 */
			x = (v >> 10);
			pommax2_send(x & 0x3f);
			csum += (x & 0x3f);

			x = (v >> 4);
			pommax2_send(x & 0x3f);
			csum += (x & 0x3f);

		}
		pommax2_send((~csum) & 0x3f);
		pommax2_send(POMMAX2_STOP);
	}
}
